// ClientProtocol.java
//
// Informatics 45 Spring 2010
// Code Example: GUI with Simple Sockets

package inf45.spring2010.examples.sockets3.protocol;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Scanner;


public class ClientProtocol implements Runnable
{
	private String name;
	private String address;
	private int port;
	
	private ArrayList<ClientProtocolListener> listeners;
	
	private Socket socket;
	
	
	public ClientProtocol(String name, String address, int port)
	{
		this.name = name;
		this.address = address;
		this.port = port;
		
		listeners = new ArrayList<ClientProtocolListener>();
		
		socket = null;
	}

	
	public void addClientProtocolListener(ClientProtocolListener listener)
	{
		listeners.add(listener);
	}
	
	
	public void removeClientProtocolListener(ClientProtocolListener listener)
	{
		if (listeners.contains(listener))
		{
			listeners.remove(listener);
		}
	}
	
	
	public void run()
	{
		socket = null;
		
		try
		{
			socket = new Socket(address, port);
			
			fireConnectionEstablished(address, port);
			
			Scanner socketIn = new Scanner(socket.getInputStream());

			PrintWriter socketOut = new PrintWriter(
				socket.getOutputStream(), true);

			socketOut.println("HELLO " + name);
			
			String okayLine = socketIn.nextLine();
			
			if (okayLine.startsWith("OKAY "))
			{
				String otherName = okayLine.substring("OKAY ".length());
				fireOkayReceived(otherName);
			}
			else
			{
				throw new ClientProtocolException("Server must send OKAY");
			}
		}
		catch (Exception e)
		{
			fireProtocolFailed(e);
		}
		finally
		{
			close();
			fireConnectionClosed();
		}
	}
	
	
	public void close()
	{
		if (socket != null)
		{
			try
			{
				socket.close();
			}
			catch (IOException ee)
			{
			}
		}
		
		socket = null;
	}
	
	
	private void fireConnectionEstablished(String serverAddress, int serverPort)
	{
		for (ClientProtocolListener listener : listeners)
		{
			listener.connectionEstablished(serverAddress, serverPort);
		}
	}
	
	
	private void fireOkayReceived(String otherName)
	{
		for (ClientProtocolListener listener : listeners)
		{
			listener.okayReceived(otherName);
		}
	}
	
	
	private void fireConnectionClosed()
	{
		for (ClientProtocolListener listener : listeners)
		{
			listener.connectionClosed();
		}
	}
	
	
	private void fireProtocolFailed(Exception failureReason)
	{
		for (ClientProtocolListener listener : listeners)
		{
			listener.protocolFailed(failureReason);
		}
	}
}
